import scrapy
import re
import logging
from crawl.comm.football import FTMatch, FTPrice, FTResult, WDLOdds, WDLSpreadOdds, ScoreResult, PointsResult, DoubleResult

class FootballSpider(scrapy.Spider):
  name = 'football'

  # 所有比赛
  _matchesMap = {}

  def start_requests(self):
    # 受注赛事
    url = 'https://info.sporttery.cn/football/match_list.php'
    yield scrapy.Request(url, self.parseMatch, 'GET')
    
  def parseMatch(self, response):
    cssOfMatches = '.all-wrap > .match_list .m-tab tr'

    # 所有赛事
    matches = response.css(cssOfMatches)
    # 当前赛事
    for node in matches:
      tdNodeList = node.css('td')

      matchWeek = tdNodeList[0].css('::text').get() # 赛事编号
      league = tdNodeList[1].css('::text').get()  # 联赛简称
      leagueFullName = tdNodeList[1].attrib.get('title')  # 联赛全称
      leagueName = '|'.join((league if league is not None else '', leagueFullName if leagueFullName is not None else ''))
      homeTeam = tdNodeList[2].css('.zhu::text').get() # 主队
      awayTeam = tdNodeList[2].css('.ke::text').get() # 客队
      matchTime = tdNodeList[3].css('::text').get() # 比赛时间
      saleStatus = tdNodeList[5].css('::text').get() # 销售状态
      saleStatus = saleStatus.strip() if saleStatus is not None else ''
      wdl = self.parsePassWay(tdNodeList[6].css('div'))
      wdls = self.parsePassWay(tdNodeList[7].css('div'))
      score = self.parsePassWay(tdNodeList[8].css('div'))
      points = self.parsePassWay(tdNodeList[9].css('div'))
      double = self.parsePassWay(tdNodeList[10].css('div'))
      
      matchLink = tdNodeList[2].css('a').attrib.get('href')
      matchId = re.sub(r'^.*m=', '', matchLink)

      # 入库
      match = FTMatch(
        matchWeek,
        matchTime,
        leagueName,
        homeTeam,
        awayTeam,
        saleStatus,
        wdl,
        wdls,
        score,
        points,
        double
      )
      match.persist()

      # 加入到临时字典里
      self._matchesMap[matchId] = match

      # 请求比赛赔率
      if saleStatus == '已开售':
        priceAPI = 'https://i.sporttery.cn/api/fb_match_info/get_pool_rs?mid=' + matchId
        yield scrapy.Request(priceAPI, self.parsePrice, 'GET')

  def parsePassWay(self, div):
    # 待开售
    if div is None:
      return 'wait'

    # 地址
    cls = div.attrib.get('class')
    if cls is None:
      return 'wait'

    # 开售单关方式和过关方式
    if "u-dan" in cls:
      return 'pass&single'

    # 仅开售过关方式
    elif "u-cir" in cls:
      return 'pass'

    # 未开售此玩法
    else:
      return 'wait'

  def parsePrice(self, response):
    logging.info("采集数据源 ---> %s" % response.url)
    data = response.json()
    res = data.get('result').get('odds_list')

    matchId = re.sub(r'^.*get_pool_rs\?mid=', '', response.url)
    match = self._matchesMap[matchId]

    ftprice = FTPrice(
      '', # 不需要 matchId
      match.matchWeek,
      match.matchTime,
      match.league,
      match.homeTeam,
      match.awayTeam
    )

    try:
      wdl = res.get('had').get('odds')[-1]
      wdlOdds = WDLOdds(wdl.get('h'), wdl.get('d'), wdl.get('a'))
      ftprice.wdlOdds = wdlOdds
    finally:
      pass

    try:
      wdls = res.get('hhad').get('odds')[-1]
      wdlsOdds = WDLSpreadOdds(wdls.get('h'), wdls.get('d'), wdls.get('a'), wdls.get('goalline'))
      ftprice.wdlsOdds = wdlsOdds
    finally:
      pass

    try:
      score = res.get('crs').get('odds')[-1]
      scoreOdds = ScoreResult(
        score.get('0100'),
        score.get('0200'),
        score.get('0201'),
        score.get('0300'),
        score.get('0301'),
        score.get('0302'),
        score.get('0400'),
        score.get('0401'),
        score.get('0402'),
        score.get('0500'),
        score.get('0501'),
        score.get('0502'),
        score.get('-1-h'),
        score.get('0000'),
        score.get('0101'),
        score.get('0202'),
        score.get('0303'),
        score.get('-1-d'),
        score.get('0001'),
        score.get('0002'),
        score.get('0102'),
        score.get('0003'),
        score.get('0103'),
        score.get('0203'),
        score.get('0004'),
        score.get('0104'),
        score.get('0204'),
        score.get('0005'),
        score.get('0105'),
        score.get('0205'),
        score.get('-1-a')
      )
      ftprice.scoreResult = scoreOdds
    finally:
      pass

    try:
      points = res.get('ttg').get('odds')[-1]
      pointsOdds = PointsResult(
        points.get('s0'),
        points.get('s1'),
        points.get('s2'),
        points.get('s3'),
        points.get('s4'),
        points.get('s5'),
        points.get('s6'),
        points.get('s7')
      )
      ftprice.pointsResult = pointsOdds
    finally:
      pass

    try:
      double = res.get('hafu').get('odds')[-1]
      doubleOdds = DoubleResult(
        double.get('hh'),
        double.get('hd'),
        double.get('ha'),
        double.get('dh'),
        double.get('dd'),
        double.get('da'),
        double.get('ah'),
        double.get('ad'),
        double.get('aa')
      )
      ftprice.doubleResult = doubleOdds
    finally:
      pass

    #
    logging.info("采集到数据 --> %s" % ftprice.toString())

    # 入库
    ftprice.persist()
